pub fn bounded_blocking<T: Send + 'static + Unpin>(
size: usize,
) -> (MTx<T>, MRx<T>)Available on crate feature
compat only.Expand description
Creates a bounded channel for use in a blocking context.
As a special case, a channel size of 0 is not supported and will be treated as a channel of size 1.